import 'package:equatable/equatable.dart';

/// Version of the touch events format expected by the native API
const int version = 1;

/// Types of the different gesture supported by the Zoning feature
enum ContentsquareGesture {
  /// pageView - 4
  ///
  /// This property is only used for the webviews tracking feature
  /// To send a PageView event with the dart SDK, use [Contentsquare.send]
  pageView,

  /// tap - 6
  tap,

  /// longsPress - 8
  longPress,

  /// drag - 9
  drag,

  /// flick - 10
  flick,
}

/// Direction of a CSGesture
/// this concerns the Swipe (Drag/Flick) : UP, DOWN, LEFT, RIGHT
enum ContentsquareGestureDirection {
  /// up - 1
  up,

  /// down - 2
  down,

  /// left - 3
  left,

  /// right - 4
  right,
}

/// Struct to hold gesture related data
class ContentsquareGestureData extends Equatable {
  /// int - dragged/scrolled distance
  final int distance;

  /// CSGestureDirection (mapped to int) -> finger direction
  /// [1,2,3,4] -> [up, down, left, right]
  ///
  /// See also:
  ///   * [ContentsquareGesture]
  final ContentsquareGestureDirection direction;

  /// int - velocity while scrolling
  final int velocity;

  /// Constructor for the CSGestureData data struct
  const ContentsquareGestureData({
    required this.distance,
    required this.direction,
    required this.velocity,
  });

  @override
  List<Object> get props => [distance, direction, velocity];
}

/// Struct to hold data for the touch events
class TouchEventData extends Equatable {
  /// String - target path inside the flutter widgets hierarchy
  final String path;

  /// CSGestureData - optional gesture related data
  final ContentsquareGestureData? gestureData;

  /// Constructor for the TouchEventData data struct
  const TouchEventData({
    required this.path,
    this.gestureData,
  });

  /// Returns a [TouchEventData] which path has been appended [prefix].
  TouchEventData copyWithPrefixedPath(String prefix) {
    return TouchEventData(
      path: prefix + path,
      gestureData: gestureData,
    );
  }

  @override
  List<Object?> get props => [path, gestureData];
}
